@@ -1,7 +1,9 @@
+
 #-*- encoding: utf-8 -*-
-VERSION='0.3.9'
+VERSION='0.4.3'
 MAX_RETRIES=5
 import sys
+import logging
 try:
     from urllib.request import urlopen, Request
     from urllib.parse import quote_plus
@@ -66,44 +68,52 @@ class JSONScraper(object):
             raise TwitchException(TwitchException.JSON_ERROR)
 
 class M3UPlaylist(object):
-    def __init__(self, input):
+    def __init__(self, input, qualityList = None):
+        self.playlist = dict()
+        self.qualityList = qualityList or Keys.QUALITY_LIST_STREAM
+
         def parseQuality(ExtXMediaLine,ExtXStreamInfLine,Url):
+            #find name of current quality, NAME=", 6 chars
             namePosition = ExtXMediaLine.find('NAME')
             if(namePosition==-1):
                 raise TwitchException()
-            qualityName = ''
+            qualityString = ''
             namePosition+=6
             for char in ExtXMediaLine[namePosition:]:
                 if(char=='"'):
                     break
-                qualityName += char
-            return qualityName, Url
+                qualityString += char
+            return qualityString, Url
 
-        self.playlist = dict()
         lines = input.splitlines()
         linesIterator = iter(lines)
         for line in linesIterator:
             if(line.startswith('#EXT-X-MEDIA')):
                 quality, url = parseQuality(line, next(linesIterator), next(linesIterator))
-                self.playlist[quality] = url
+                qualityInt = self.qualityList.index(quality)
+                self.playlist[qualityInt] = url
+        if not self.playlist:
+            #playlist dict is empty
+            raise ValueError('could not find playable urls')
 
     #returns selected quality or best match if not available
-    def getQuality(self, QualityInt):
-        def isInPlaylist(QualityInt):
-            return Keys.QUALITY_LIST_STREAM[QualityInt] in self.playlist
-
-        if(isInPlaylist(QualityInt)):
+    def getQuality(self, selectedQuality):
+        if(selectedQuality in self.playlist.keys()):
             #selected quality is available
-            return self.playlist[Keys.QUALITY_LIST_STREAM[QualityInt]]
+            return self.playlist[selectedQuality]
         else:
-            #not available, start with worst quality and improve
-            #break if better quality is not available
-            bestMatch = len(Keys.QUALITY_LIST_STREAM) - 1
-            for newMatch in range(bestMatch, -1, -1):
-                if(isInPlaylist(newMatch)):
-                    bestMatch = newMatch
+            #not available, calculate differences to available qualities
+            #return lowest difference / lower quality if same distance
+            bestDistance = len(self.qualityList) + 1
+            bestMatch = None
 
-            return self.playlist[Keys.QUALITY_LIST_STREAM[bestMatch]]
+            for quality in sorted(self.playlist, reverse=True):
+                newDistance = abs(selectedQuality - quality)
+                if newDistance < bestDistance:
+                    bestDistance = newDistance
+                    bestMatch = quality
+
+            return self.playlist[bestMatch]
 
     def __str__(self):
         return repr(self.playlist)
@@ -113,7 +123,7 @@ class TwitchTV(object):
     Uses Twitch API to fetch json-encoded objects
     every method returns a dict containing the objects\' values
     '''
-    def __init__(self, logger):
+    def __init__(self, logger=logging):
         self.logger = logger
         self.scraper = JSONScraper(logger)
 
@@ -155,6 +165,21 @@ class TwitchTV(object):
         channels['others'] = channelNames
         return channels
 
+    def getFollowingGames(self, username):
+        acc = []
+        limit = 100
+        offset = 0
+        quotedUsername = quote_plus(username)
+        baseurl = Urls.FOLLOWED_GAMES.format(quotedUsername)
+        while True:
+            url = baseurl + Urls.OPTIONS_OFFSET_LIMIT.format(offset, limit)
+            temp = self._fetchItems(url, Keys.FOLLOWS)
+            if (len(temp) == 0):
+                break;
+            acc = acc + temp
+            offset = offset + limit
+        return acc
+
     def getFollowerVideos(self, username, offset, past):
         url = Urls.CHANNEL_VIDEOS.format(username,offset,past)
         items = self.scraper.getJson(url)
@@ -313,8 +338,9 @@ class Keys(object):
     PREVIEW = 'preview'
     TITLE = 'title'
 
-    QUALITY_LIST_STREAM = ['Source', "High", "Medium", "Low", "Mobile"]
-    QUALITY_LIST_VIDEO = ['live', "720p", "480p", "360p", "226p"]
+    QUALITY_LIST_STREAM = ['Source', 'High', 'Medium', 'Low', 'Mobile']
+    QUALITY_LIST_VIDEO = ['live', '720p', '480p', '360p', '226p']
+
 
 class Urls(object):
     '''
@@ -342,6 +368,7 @@ class Urls(object):
     CHANNEL_VIDEOS = 'https://api.twitch.tv/kraken/channels/{0}/videos?limit=8&offset={1}&broadcasts={2}'
     VIDEO_PLAYLIST = 'https://api.twitch.tv/api/videos/{0}'
     VIDEO_INFO = 'https://api.twitch.tv/kraken/videos/{0}'
+    FOLLOWED_GAMES = 'https://api.twitch.tv/api/users/{0}/follows/games'
 
 
 class TwitchException(Exception):
